import React, { useState, useEffect } from 'react';
import { getAvailability } from './api';

const NameForm = ({ onNameSet }) => {
  const [name, setName] = useState("");

  return (
    <div>
      <form onSubmit={() => onNameSet(name)}>
        <input 
          type="text" 
          onChange={({ target }) => setName(target.value)}
          value={name}
        />
        <input 
          disabled={name === ""} 
          type="submit" 
          value="Submit"
        />
      </form>
    </div>
  )
}

const colors = {
  yes: "#0f0",
  no: "#f00",
  maybe: "#ff0",
}

const nextState = {
  yes: "no",
  no: "maybe",
  maybe: "unknown",
  unknown: "yes",
}

const pad = num => {
  let snum = "" + num
  if (snum.length === 1) snum = "0" + snum
  return snum
}

const Tile = ({ user, month, day, availability, onClick }) => (
  <div 
    onClick={onClick}
    style={{
      display: "flex",
      flexFlow: "column nowrap",
      width: "5em",
      height: "5em",
      border: "1px solid black",
      margin: "2px 2px 2px 2px",
      backgroundColor: colors[availability.filter(({ name }) => name === user)[0]?.status] || "#fff",
    }}
  >
    <span>{pad(month)}/{pad(day)}</span>
    {
      availability
        .filter(({ status }) => status && status !== "unknown")
        .filter(({ name }) => name !== user)
        .map(({ name, status }) => (
          <span 
            key={name} 
            style={{ color: colors[status] }}
          >
            {name}
          </span>
        ))
      }
  </div>
);


const chunk = (arr, len) => {
  const chunks = [];
  for (let i = 0; i < arr.length;) {
    chunks.push(arr.slice(i, i += len));
  }
  return chunks;
}

const advanceDay = (dates, name, month, day) => {
  const dateInd = dates.findIndex(item => item.month === month && item.day === day);
  if (dateInd === -1) {
    return dates;
  }
  const leftDates = dates.slice(0, dateInd);
  const date = dates[dateInd];
  const rightDates = dates.slice(dateInd + 1);
  const avInd = date.availability.findIndex(item => item.name === name);

  let otherAvail;
  let status;
  if (avInd === -1) {
    otherAvail = date.availability;
    status = nextState.unknown;
  } else {
    const leftAvail = date.availability.slice(0, avInd);
    const rightAvail = date.availability.slice(avInd + 1);
    otherAvail = [...leftAvail, ...rightAvail];
    status = nextState[date.availability[avInd].status];
  }
  return [...leftDates, {...date, availability: [{ name, status }, ...otherAvail]}, ...rightDates];
}

const advanceWeek = (dates, name, dateRange) => dateRange.reduce((acc, { month, day }) => advanceDay(acc, name, month, day), dates)

function App() {
  const [name, setName] = useState(null);
  const [avail, setAvail] = useState(null);
  useEffect(() => { getAvailability().then(({availability}) => setAvail(availability)) }, []);


  if (name === null) {
    return <NameForm onNameSet={setName} />
  }

  if (avail === null) {
    return <span>Loading...</span>
  }

  return (
    <div
      style={{
        display: "flex",
        flexFlow: "column nowrap",
      }}
    >
      <span>I will plan this vacation if it kills me.</span>
      {
        chunk(avail, 7).map((row, ind) => (
          <div
            key={ind} 
            style={{
              display: "flex",
              flexFlow: "row nowrap",
              alignItems: "center",
            }}
          >
            <div
              onClick={() => setAvail(advanceWeek(avail, name, row))}
            >
              WEEK
            </div>
            <div 
              style={{
                display: "flex",
                flexFlow: "row nowrap",
              }}
            >
              {
                row.map(item => <Tile onClick={() => setAvail(advanceDay(avail, name, item.month, item.day))} key={item.day} user={name} {...item}/>)
              }
            </div>
          </div>
        ))
      }
    </div>
  );
}

export default App;
